Package: GCPtools
Title: Tools for working with gcloud and gsutil
Version: 1.3.2
Date: 2026-05-22
Authors@R: c(
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu",
        c("aut", "cre"), c(ORCID = "0000-0002-3242-0582")
    ),
    person("Nitesh", "Turaga", role = "aut"),
    person("Martin", "Morgan", , , "aut", c(ORCID = "0000-0002-5874-8148"))
    )
Description: Lower-level functionality to interface with Google Cloud Platform
    tools. 'gcloud' and 'gsutil' are both supported. The functionality provided
    centers around utilities for the AnVIL platform.
biocViews: Software, Infrastructure, ThirdPartyClient, DataImport
SystemRequirements: gsutil, gcloud
Depends: R (>= 4.6.0)
Imports: AnVILBase, BiocBaseUtils, dplyr, httr, rlang, tibble, tidyr,
        utils
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0)
License: Artistic-2.0
URL: https://github.com/Bioconductor/GCPtools
BugReports: https://github.com/Bioconductor/GCPtools/issues
Encoding: UTF-8
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/GCPtools
git_branch: devel
git_last_commit: ee812c9
git_last_commit_date: 2026-05-22
Repository: Bioconductor 3.24
Date/Publication: 2026-05-24
NeedsCompilation: no
Packaged: 2026-05-24 21:58:08 UTC; biocbuild
Author: Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Nitesh Turaga [aut],
  Martin Morgan [aut] (ORCID: <https://orcid.org/0000-0002-5874-8148>)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
