# Generated by roxygen2: do not edit by hand

S3method(print,gcloud_sdk_result)
export(gcloud_access_token)
export(gcloud_account)
export(gcloud_cmd)
export(gcloud_exists)
export(gcloud_help)
export(gcloud_project)
export(gcloud_storage)
export(gcloud_storage_buckets)
export(gsutil_cat)
export(gsutil_cp)
export(gsutil_exists)
export(gsutil_help)
export(gsutil_is_uri)
export(gsutil_ls)
export(gsutil_pipe)
export(gsutil_requesterpays)
export(gsutil_requesterpays_flag)
export(gsutil_rm)
export(gsutil_rsync)
export(gsutil_sh_quote)
export(gsutil_stat)
importFrom(AnVILBase,avstop_for_status)
importFrom(BiocBaseUtils,isCharacter)
importFrom(BiocBaseUtils,isScalarCharacter)
importFrom(BiocBaseUtils,isScalarLogical)
importFrom(BiocBaseUtils,isZeroOneCharacter)
importFrom(httr,POST)
importFrom(httr,content)
importFrom(httr,content_type)
importFrom(rlang,.data)
importFrom(tidyr,pivot_wider)
importFrom(utils,tail)
